import { useState, useEffect } from 'react';
import styled from 'styled-components';
import { Table } from 'react-bootstrap';
import axios from 'axios';

import { List } from '@/common/components/List';
import { ListHeader, ListHeaders } from '@/common/components/List/ListHeader';
import { TableListItem } from '@/common/components/List';
import {
  HeaderText,
  SortIconDown,
  SortIconUp,
} from '@/common/components/SortedListHeaders';
import { TextBig, TokenValue } from '@/common/components/typography';

import Loading from '../Loading';
import './bounties.css';

interface IBounty {
  id: any;
  title: string;
  description: string;
  format: string;
  openedDate: any;
  status: string;
  reward: string | number;
  manager: string[];
  links: string[];
}

interface ITableColumn {
  field: string;
  title: string;
  format?: Function;
}

const HEADERS: ITableColumn[] = [
  {
    field: 'id',
    title: 'ID',
  },
  {
    field: 'links',
    title: 'Links',
    format: (links: string[]) => links && links.map(l => <>{l}<br /></>)
  },
  {
    field: 'reward',
    title: 'Reward',
  },
  {
    field: 'manager',
    title: 'Manager, Co',
  },
  {
    field: 'status',
    title: 'Status/Grading',
  },
];

const BountyListItem = ({ bounty }: { bounty: IBounty }) => {
  return (
    <TableListItem>
      <TextBig as="h5" bold>
        {bounty.id}
      </TextBig>

      <TextBig as="h5" bold>
        {bounty.title}
      </TextBig>
    </TableListItem>
  );
};

const UniversalTableItem = ({ children }) => {
  return (
    <TextBig as="h5" bold>
      {children}
    </TextBig>
  );
};

// 10px 200px 200px 250px 240px
const UniversalTableListItem = styled(TableListItem)/* .attrs({ $colLayout: '' }) */`
  grid-template-columns: repeat(5, auto);
  justify-content: left;
  gap: 20px;
`;

const UniversalTableListHeaders = styled(ListHeaders)`
  grid-template-columns: repeat(5, auto);
  justify-content: left;
  gap: 20px;
`;

const UniversalTable = ({
  columns,
  data,
}: {
  columns: ITableColumn[];
  data: any[];
}) => {
  return (
    <>
      <UniversalTableListHeaders /* $colLayout={CouncilColLayout} */>
        {columns.map(({ title, field }, index) => (
          <ListHeader key={field}>
            <HeaderText>
              {title}
              {/* {order.key === key && (order.isDescending ? <SortIconDown /> : <SortIconUp />)} */}
            </HeaderText>
          </ListHeader>
        ))}
      </UniversalTableListHeaders>
      <List>
        {data &&
          data.map((item: any, index: number) => {
            return (
              <UniversalTableListItem>
                {columns.map(({ field, format }) => (
                  <UniversalTableItem key={field}>
                    {format && format(item[field])}
                    {!format && item[field]}
                  </UniversalTableItem>
                ))}
              </UniversalTableListItem>
            );
          })}
      </List>
    </>
  );
};

function Bounties() {
  const [bounties, setBounties] = useState<IBounty[] | null>(null);

  const fetchBounties = async () => {
    const response = await axios.get<{ activeBounties: IBounty[] }>(
      `https://raw.githubusercontent.com/Joystream/community-repo/master/bounties/overview/bounties-status.json`
    );

    setBounties(response.data?.activeBounties);
  };

  useEffect(() => {
    fetchBounties();
  }, []);

  if (!bounties) return <Loading target={`bounties`} />;

  return (
    <>
      <div className="Bounties m-2 p-1 bg-light">
        <h1>Bounties</h1>
        <UniversalTable columns={HEADERS} data={bounties} />
        <br />
        <br />
        <br />
        <ListHeaders>
          {HEADERS.map(({ title, field }, index) => (
            <ListHeader key={field}>
              <HeaderText>
                {title}
                {/* {order.key === key && (order.isDescending ? <SortIconDown /> : <SortIconUp />)} */}
              </HeaderText>
            </ListHeader>
          ))}
        </ListHeaders>
        <List>
          {bounties &&
            bounties.map((bounty, index) => (
              <BountyListItem key={index} bounty={bounty} />
            ))}
        </List>
        <br />
        <br />
        <br />
        <br />
        <hr />
        <Table striped bordered hover>
          <thead>
            <tr>
              <th>#</th>
              <th>Title</th>
              <th>Description</th>
              <th>Links</th>
              <th>Opened date</th>
              <th>Reward</th>
              <th>Manager, Co</th>
              <th>Status/Grading</th>
            </tr>
          </thead>
          <tbody>
            {bounties && bounties.map((b: any) => <Bounty key={b.id} {...b} />)}
          </tbody>
        </Table>
        <a href="https://github.com/Joystream/community-repo/tree/master/workinggroup-reports/bounty_reports">
          Reports
        </a>
        <br />
        <a href="https://github.com/Joystream/community-repo/blob/master/bounties-overview/README.md">
          Closed Bounties
        </a>
      </div>
    </>
  );
}
export default Bounties;

const Bounty = ({
  id,
  title,
  description,
  format,
  openedDate,
  status,
  reward,
  manager,
  links,
}: IBounty) => {
  return (
    <tr>
      <td>{id}</td>
      <td>{title}</td>
      <td>{description}</td>
      <td>
        {links.map((l) => (
          <>
            <a href={l}>{l}</a>
            <br />
          </>
        ))}
      </td>
      <td>{openedDate}</td>
      <td>${reward}</td>
      <td>{manager?.join(', ')}</td>
      <td>{status}</td>
    </tr>
  );
};
