import { Table } from "@material-ui/core";
import styled from "styled-components";
import React from "react";
import { Link } from "react-router-dom";
import { Stakes } from "../../types";

import {
  BlockDurationStatistics,
  MultiValueStat,
  Statistics,
} from "@/common/components/statistics";
import {
  MultiStatisticItem,
  StatsBlock,
  StatisticItem,
  StatisticLabel,
  StatisticItemProps,
  StatisticItemSpacedContent,
  StatisticHeader,
  StatisticHeaderProps,
  TotalValue,
} from "@/common/components/statistics";

import { DurationValue } from "@/common/components/typography/DurationValue";
import { BlockIcon, BlockIconStyles } from "@/common/components/icons";
import { TextInlineSmall } from "@/common/components/typography";
import { ColumnGapBlock } from "@/common/components/page/PageContent";
import { AN_HOUR, A_DAY, A_MINUTE, A_WEEK, Fonts } from "@/common/constants";

const dollar = (d: number) => (d > 0 ? `$ ${d.toFixed(2)}` : "");

const MinMax = (props: {
  stakes?: { [key: string]: Stakes };
  block: number;
  era: number;
  issued: number;
  validators: string[];
  nominators: number;
  waiting: number;
  reward: number;
  price: number;
}) => {
  const classes = useStyles();
  const { issued, stakes, validators, waiting, reward, price } = props;
  if (!stakes || !Object.values(stakes).length) return <span />;

  let sum = 0;
  let minStake: number = 10000000;
  let maxStake: number = 0;
  Object.keys(stakes).forEach((v: string) => {
    if (!validators.includes(v)) return;
    const { total } = stakes[v];
    if (total > 0) sum += total;
    else return;

    if (total > maxStake) maxStake = total;
    if (total < minStake) minStake = total;
  });

  const name = { className: "text-right" };
  const value = { className: "text-left" };
  const validatorReward = reward ? reward / validators.length : 0;

  const validatorStats = [
    {
      label: "Validators",
      value: validators.length,
    },
    {
      label: "Waiting",
      value: waiting,
    },
    {
      label: "Nominators",
      value: props.nominators,
    },
    {
      label: "Issued",
      value: <TotalValue value={`${(issued / 1000000).toFixed(1)} M`} />,
    },
    {
      label: "Min stake",
      value: <TotalValue value={minStake} />,
    },
    {
      label: "Max stake",
      value: <TotalValue value={maxStake} />,
    },
    {
      label: "Total payed per hour",
      value: (
        <>
          <TotalValue value={reward ? `${reward} JOY ` : "Loading.."} />(
          {dollar(price * reward)})
        </>
      ),
    },
    // {
    //   label: 'Validators',
    //   value: validator,
    // },
  ];

  // https://pioneer-2.netlify.app/#/working-groups/forum
  // https://pioneer-2.netlify.app/#/council
  // https://joystreamstats.live/
  return (
    <>
      <Statistics>
        {/* <StatisticItem title="Validators" size="s"> */}
        <StatsBlock size="s">
          <StatisticHeader title="Validators" />
          <StatsContent>
            {validatorStats.map((v) => (
              <StatisticItemSpacedContent key={v.label}>
                <StatisticLabel>{v.label}</StatisticLabel>
                {v.value}
              </StatisticItemSpacedContent>
            ))}
          </StatsContent>
        </StatsBlock>
        <MultiStatisticItem title="Validators">
          <ItemRow>
            <DurationValue value={validators.length} />
          </ItemRow>

          <ItemRow>
            <BlocksInfo gap={8}>
              <BlockIcon />
              <NumberOfBlocks lighter>
                {/* {formatTokenValue(duration)} block{plural(props.value)} */}
              </NumberOfBlocks>
            </BlocksInfo>
          </ItemRow>
        </MultiStatisticItem>
      </Statistics>
      {/* <Table
        style={{
          textAlign: 'center',
          backgroundColor: '#4038FF',
          color: '#fff',
        }}
      >
        <tbody>
          <tr>
            <td {...name}>Validators</td>
            <td {...value}>{validators.length}</td>
          </tr>
          <tr>
            <td {...name}>Waiting</td>
            <td {...value}>{waiting}</td>
          </tr>
          <tr>
            <td {...name}>Nominators</td>
            <td {...value}>{props.nominators}</td>
          </tr>

          <tr>
            <td {...name}>Issued</td>
            <td {...value}> {(issued / 1000000).toFixed(1)} M JOY</td>
          </tr>
          <tr>
            <td {...name}>Staked</td>
            <td {...value}>
              {(sum / 1000000).toFixed(1)} M JOY (
              <b>{issued ? ((sum / issued) * 100).toFixed(1) : `-`}%</b>)
            </td>
          </tr>
          <tr>
            <td {...name}>Min stake</td>
            <td {...value}>{minStake} JOY</td>
          </tr>
          <tr>
            <td {...name}>Max stake</td>
            <td {...value}>{maxStake} JOY </td>
          </tr>
          <tr>
            <td {...name}>Total payed per hour</td>
            <td {...value}>
              {reward
                ? `${reward} JOY (${dollar(price * reward)})`
                : 'Loading..'}
            </td>
          </tr>
          <tr>
            <td {...name}>Reward per validator per hour</td>
            <td className="text-left text-warning">
              <ValidatorReward
                show={reward}
                price={price}
                reward={validatorReward}
              />
            </td>
          </tr>
        </tbody>
      </Table> */}
    </>
  );
};
export default MinMax;

const ValidatorReward = (props) => {
  const { show, price, reward } = props;
  if (!show) return `Loading..`;
  return (
    <>
      {reward.toFixed(0)} JOY ({dollar(price * reward)})
      <Link className="ml-1" to={"/mint"}>
        Details
      </Link>
    </>
  );
};

const BlocksInfo = styled(ColumnGapBlock)`
  margin-top: 2px;
`;

const ItemRow = styled(StatisticItemSpacedContent)`
  justify-content: start;
`;

const NumberOfBlocks = styled(TextInlineSmall)`
  font-family: ${Fonts.Grotesk};
  ${BlockIconStyles} {
    vertical-align: middle;
  }
`;

export const StatsContent = styled.div`
  margin-top: auto;
`;

// export const MultiStatisticItem = styled(StatisticItem)<{ moreInnerSpace?: boolean }>`
//   padding: ${({ moreInnerSpace }) => (moreInnerSpace ? '12px 16px 4px' : '12px 16px 16px')};
//   ${(props: any) => StatsBlock(props)}
// `
